package info.bukova.isspst;

import info.bukova.isspst.data.Permission;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.reporting.Report;
import info.bukova.isspst.reporting.ReportMapping;
import info.bukova.isspst.reporting.ReportType;
import info.bukova.isspst.services.users.PermissionService;
import info.bukova.isspst.services.users.RoleService;
import info.bukova.isspst.services.users.UserService;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

public class AppInitListener implements ServletContextListener {

	private RoleService roleService;
	private UserService userService;
	private PermissionService permService;
	
	@Override
	public void contextDestroyed(ServletContextEvent arg0) {

	}

	@Override
	public void contextInitialized(ServletContextEvent evt) {
		Logger logger = LoggerFactory.getLogger(AppInitListener.class);
		logger.info("Initializing database");
		
		WebApplicationContext ctx = WebApplicationContextUtils.getRequiredWebApplicationContext(evt.getServletContext());
		roleService = ctx.getBean(RoleService.class);
		userService = ctx.getBean(UserService.class);
		permService = ctx.getBean(PermissionService.class);
		
		userService.grantAdmin();
		checkRoles();
		checkUsers();
		checkPermissions();
		userService.removeAccess();
		
		loadModuleReports();
	}
	
	private void checkRoles() {
		for (Role role : Constants.ROLES)
		{
			if (roleService.getRoleByAuthority(role.getAuthority()) == null) {
				roleService.add(role);
			}
		}
	}
	
	private void checkUsers() {
		boolean adminExists = false;
		
		for (User u : userService.getAll()) {
			if (userService.hasRole(u, Constants.ROLE_ADMIN)) {
				adminExists = true;
				break;
			}
		}
		
		if (!adminExists) {
			User u = null;
			try {
				u = (User) userService.loadUserByUsername(Constants.DEF_ADMIN);
				u.addAuthority(roleService.getRoleByAuthority(Constants.ROLE_ADMIN));
				u.setEnabled(true);
				userService.update(u);
			} catch(UsernameNotFoundException e) {
				u = new User();
				u.setUsername(Constants.DEF_ADMIN);
				userService.setPassword(u, Constants.DEF_ADMIN_PASSWD);
				u.addAuthority(roleService.getRoleByAuthority(Constants.ROLE_ADMIN));
				u.setEnabled(true);
				userService.add(u);
			}
		}
	}
	
	private void checkPermissions() {
		for (Module m : Constants.MODULES) {
			for (Permission p : Constants.DEF_PERMISSIONS) {
				if (permService.getPermissionByModule(m.getId(), p.getAuthority()) == null) {
					p.setModule(m.getId());
					permService.add(p);
				}
			}
		}
	}
	
	private void loadModuleReports() {
		for (Module m : Constants.MODULES) {
			Report rep = new Report();
			rep.setType(ReportType.DYNAMIC);
			rep.setName(Constants.DYNAMIC_REPORT_NAME);
			m.addReport(rep);
			for (ReportMapping r : Constants.REPORTS) {
				if (m.getId().equals(r.getModule())) {
					m.addReport(r.getReport());
				}
			}
		}
	}

}
