package info.bukova.isspst.security;

import info.bukova.isspst.Constants;
import info.bukova.isspst.Module;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.services.Service;

import java.io.Serializable;
import java.util.List;

import org.springframework.security.access.PermissionEvaluator;
import org.springframework.security.core.Authentication;

public class IsspstPermissionEvaluator implements PermissionEvaluator {

	@SuppressWarnings("unchecked")
	@Override
	public boolean hasPermission(Authentication authentication,
			Object targetDomainObject, Object permission) {
		List<Role> perms = (List<Role>) authentication.getAuthorities();
		String moduleId = "";
		String perm = "";
		
		if (permission instanceof String) {
			perm = (String) permission;
		}
		
		if (targetDomainObject instanceof Service<?>)
		{
			for (Module m : Constants.MODULES) {
				if (m.getServiceClass().isAssignableFrom(targetDomainObject.getClass())) {
					moduleId = m.getId();
				}
			}
			
			perm += "_" + moduleId;
			
			for (int i = 0; i < perms.size(); i++) {
				if (!(perms.get(i) instanceof Role)) {
					return false;
				}
				if (perms.get(i).getAuthority().equals(perm)) {
					return true;
				}
				if (perms.get(i).getAuthority().equals(Constants.ROLE_ADMIN)) {
					return true;
				}
			}
		}
		
		return false;
	}

	@Override
	public boolean hasPermission(Authentication authentication,
			Serializable targetId, String targetType, Object permission) {
		return false;
	}

}
