package info.bukova.isspst.services.users;

import org.hibernate.Query;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.authentication.encoding.PasswordEncoder;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.transaction.annotation.Transactional;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.services.AbstractService;

public class UserServiceImpl extends AbstractService<User> implements UserService {

	private PasswordEncoder encoder;
	
	public void setEncoder(PasswordEncoder encoder) {
		this.encoder = encoder;
	}
	
	@Override
	@Transactional
	public UserDetails loadUserByUsername(String username)
			throws UsernameNotFoundException {
		
		Query q = dao.getQuery("from User user where user.username = :username");
		q.setString("username", username);
		User u = (User) q.uniqueResult();
		
		if (u == null)
			throw new UsernameNotFoundException("User " + username + " not found");
		
		return u;
	}

	@Override
	public void setPassword(User user, String password) {
		user.setPassword(encodePassword(user, password));
	}

	@Override
	public boolean hasRole(User user, String authority) {
		for (Role r : user.getAuthorities()) {
			if (r.getAuthority().equals(authority)) {
				return true;
			}
		}
		return false;
	}

	@Override
	@Transactional
	public void saveWithPwd(User user, String password) {
		this.setPassword(user, password);
		this.update(user); 
	}

	@Override
	@Transactional
	public User getCurrent() {
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		
		if (auth != null && auth.getPrincipal() != null) {
			try {
				return (User)loadUserByUsername(((UserDetails)auth.getPrincipal()).getUsername());
			} catch(UsernameNotFoundException e) {
				return null;
			}
		}
		
		return null;
	}

	@Override
	public String encodePassword(User user, String plain) {
		return encoder.encodePassword(plain, user.getUsername());
	}

	@Override
	public void grantAdmin() {
		User tmpAdmin = new User();
		Role tmpRole = new Role();
		tmpRole.setAuthority(Constants.ROLE_ADMIN);
		tmpAdmin.setUsername(Constants.DEF_ADMIN);
		tmpAdmin.addAuthority(tmpRole);
		SecurityContextHolder.getContext().setAuthentication(new UsernamePasswordAuthenticationToken(tmpAdmin, null, tmpAdmin.getAuthorities()));
	}

	@Override
	public void removeAccess() {
		SecurityContextHolder.getContext().setAuthentication(null);
	}


}
